/////////////////////////////////////////////////////////////////////////////
// Name:        gbutton.cpp
// Purpose:     GUI button implementation using wxWidgets.
// Author:      Yann Hamiaux
// Modified by: -
// Created:     03/03/2008
// Copyright:   Written for Guillemot Corporation by Yann Hamiaux
// Licence:     The GNU General Public License v2+.
/////////////////////////////////////////////////////////////////////////////
#include "gbutton.h"

CGButton::CGButton( PEVENT_LIST pEventList, int nControlID, int nPosX, int nPosY, int nWidth, int nHeight, 
				   wxMemoryDC *pSkinMemDC, wxMemoryDC *pSkinAlphaMemDC, int nStateCount, 
				   PBUTTON_STATE_INFO pStateInfo ) :
			CGControl(pEventList, nControlID, GCONTROL_TYPE_BUTTON, nPosX, nPosY, nWidth, nHeight)
{
	int	i, j;
	m_pSkinMemDC = pSkinMemDC;
	m_pSkinAlphaMemDC = pSkinAlphaMemDC;
	m_nStateCount = nStateCount;
	m_pStateInfo = NULL;
	if (m_nStateCount>0) { 
		m_pStateInfo = new BUTTON_STATE_INFO[m_nStateCount];
		for(i=0;i<m_nStateCount;++i) {
			for(j=0;j<4;++j) {
				m_pStateInfo[i].coordinates[j].x = pStateInfo[i].coordinates[j].x;
				m_pStateInfo[i].coordinates[j].y = pStateInfo[i].coordinates[j].y;
			}
			m_pStateInfo[i].nProcessAlpha = pStateInfo[i].nProcessAlpha;
		}
	}
	m_nCurrentState = 0;
	m_nCurrentSubState = BUTTON_STATE_NORMAL;
}

CGButton::~CGButton()
{
	if (m_pStateInfo!=NULL)
		delete m_pStateInfo;
}

// ---------------------------------------------------------------------------
// Function: GetState
//
// Return how many state the button has.
// ---------------------------------------------------------------------------
int CGButton::GetState()
{
	return m_nCurrentState;
}

// ---------------------------------------------------------------------------
// Function: SetState
//
// Change button current state.
// ---------------------------------------------------------------------------
void CGButton::SetState(int nState)
{
	if (nState>=0 && nState<m_nStateCount)
		m_nCurrentState = nState;
}

// ---------------------------------------------------------------------------
// Function: Enable
//
// Enable/Disable button.
// ---------------------------------------------------------------------------
void CGButton::Enable( bool bEnable )
{
	CGControl::Enable(bEnable);
	if (!m_bEnable)
		m_nCurrentSubState = BUTTON_STATE_DISABLE;
	else
		m_nCurrentSubState = BUTTON_STATE_NORMAL;
}


// ---------------------------------------------------------------------------
// Function: Paint
//
// Paint button with or without the alpha channel.
// ---------------------------------------------------------------------------
void CGButton::Paint( wxDC *pDC, PREFRESH_RECT refreshRect )
{
	if (m_bHide || pDC==NULL || m_pSkinMemDC==NULL || m_pStateInfo==NULL || m_nCurrentState>=m_nStateCount)
		return;

	// --- If button is outside of the region, do not refresh it ---
	if (refreshRect->x>m_nPosX+m_nWidth || refreshRect->x+refreshRect->w<m_nPosX)
		return;
	if (refreshRect->y>m_nPosY+m_nHeight || refreshRect->y+refreshRect->h<m_nPosY)
		return;

	// --- Paint with or without transparency (stored in the .PNG file) ---
	if (m_pStateInfo[m_nCurrentState].nProcessAlpha) {
		pDC->Blit(m_nPosX, m_nPosY, m_nWidth, m_nHeight, m_pSkinAlphaMemDC, 
			m_pStateInfo[m_nCurrentState].coordinates[m_nCurrentSubState].x, 
			m_pStateInfo[m_nCurrentState].coordinates[m_nCurrentSubState].y, wxAND);
		pDC->Blit(m_nPosX, m_nPosY, m_nWidth, m_nHeight, m_pSkinMemDC, 
			m_pStateInfo[m_nCurrentState].coordinates[m_nCurrentSubState].x, 
			m_pStateInfo[m_nCurrentState].coordinates[m_nCurrentSubState].y, wxOR);
	}
	else
	{
		pDC->Blit(m_nPosX, m_nPosY, m_nWidth, m_nHeight, m_pSkinMemDC, 
			m_pStateInfo[m_nCurrentState].coordinates[m_nCurrentSubState].x, 
			m_pStateInfo[m_nCurrentState].coordinates[m_nCurrentSubState].y, wxCOPY);
	}
}

// ---------------------------------------------------------------------------
// Function: MouseEvent
//
// Change button sub-state base on the mouse (normal, highlight or pressed).
// ---------------------------------------------------------------------------
bool CGButton::MouseEvent( int x, int y, int button, bool *bMouseOverCtrl, 
				PREFRESH_RECT pRefreshRect )
{
	if (bMouseOverCtrl==NULL)
		return false;

	*bMouseOverCtrl = false;
	if (m_bHide || m_nCurrentSubState==BUTTON_STATE_DISABLE)
		return false;
	CGControl::MouseEvent(x,y,button,bMouseOverCtrl,pRefreshRect);

	if (*bMouseOverCtrl)
	{
		// Inside the control.
		if (button&GMOUSE_LEFT_DOWN) {
			if (m_nCurrentSubState!=BUTTON_STATE_PRESSED) {
				m_nCurrentSubState = BUTTON_STATE_PRESSED;
				return true;
			}
		}
		else {
			if (m_nCurrentSubState!=BUTTON_STATE_HIGHLIGHT) {
				if (m_nCurrentSubState==BUTTON_STATE_PRESSED) {
					m_nCurrentState = (m_nCurrentState+1)%m_nStateCount;
					SetEvent(GEVENT_BTN_CHANGE, m_nCurrentState, 0);
				}
				m_nCurrentSubState = BUTTON_STATE_HIGHLIGHT;
				return true;
			}
		}
	}
	else
	{
		// Outside the control.
		if (m_nCurrentSubState!=BUTTON_STATE_NORMAL) {
			m_nCurrentSubState = BUTTON_STATE_NORMAL;
			return true;
		}
	}
	return false;
}

