/////////////////////////////////////////////////////////////////////////////
// Name:        hidnotification.cpp
// Purpose:     wxThread derived class to handle HID notification 
//		(read pooling) via the bulk interface.
// Author:      Yann Hamiaux
// Modified by: -
// Created:     22/10/2008
// Copyright:   Written for Guillemot Corporation by Yann Hamiaux
// Licence:     The GNU General Public License v2+.
/////////////////////////////////////////////////////////////////////////////
#include "hidnotification.h"

CHIDNotification::CHIDNotification( DJ_DEVICE_STATE *pDevState )
{
	m_nBulkFileHandle = pDevState->nBULK_FHandle;
	m_ulBufferSize = pDevState->ulBULK_ReadSize;
	m_pDevState = pDevState;
}

CHIDNotification::~CHIDNotification()
{
}

// ---------------------------------------------------------------------------
// Function: Entry
//
// Main loop for the HID Notification thread, which consist of reading the
// HID data throught the bulk interface for the DJ Console and DJ Console Mk2.
// Further DJ products report controls and options via the "Bulk" N-point.
//
// However, for others products like DJ Control Steel or DJ Console Rmx;
// no read pooling is required for the Control Panel.
// ---------------------------------------------------------------------------
void* CHIDNotification::Entry()
{
	AUDIO_SOURCE	eAudioSourceSelection;
	HEADPHONE_SEL	eHeadphoneSelection;
	bool 		bOnAir, bLock;
	char 		*pBuf;
	int 		nRet;

	if (m_ulBufferSize<=0)
		return NULL;
	
	pBuf = new char[m_ulBufferSize];
	if (pBuf==NULL)
		return NULL;

	while(TestDestroy()!=true)
	{
		memset(pBuf, 0, m_ulBufferSize);
		nRet = read(m_nBulkFileHandle, pBuf, m_ulBufferSize);

		switch(m_pDevState->ulProductCode)
		{
		case DJCONSOLE_PRODUCT_CODE:
			// DJ Console: We get the ON AIR button state.
			bOnAir = (pBuf[DJC_BULK_INDEX_ON_AIR]&DJC_BULK_MASK_ON_AIR)!=0;	
			if (m_pDevState->djConsole.bOnAir!=bOnAir)
			{
				m_pDevState->djConsole.bOnAir = bOnAir;
				m_pDevState->bStateChange = true;
			}
			break;
		case DJCONSOLE2_PRODUCT_CODE:
			// --- On Air State ---
			bOnAir = (pBuf[DJMK2_BULK_INDEX_ON_AIR]&DJMK2_BULK_MASK_ON_AIR)!=0;
			if (bOnAir!=m_pDevState->djConsoleMk2.bOnAir)
			{
				m_pDevState->djConsoleMk2.bOnAir = bOnAir;
				m_pDevState->bStateChange = true;
			}

			// --- Audio Source Selection (Deck A and Deck B) ---
			eAudioSourceSelection = (pBuf[DJMK2_BULK_INDEX_AUDIO_SOURCE]&DJMK2_BULK_MASK_AUDIO_SOURCE_DECK_A) ?
				USB_PC_OUT : MK2_AUDIO_INPUT;
			if (eAudioSourceSelection!=m_pDevState->djConsoleMk2.eAudioSourceSelection[0]) {
				m_pDevState->djConsoleMk2.eAudioSourceSelection[0] = eAudioSourceSelection;
				m_pDevState->bStateChange = true;
			}

			eAudioSourceSelection = (pBuf[DJMK2_BULK_INDEX_AUDIO_SOURCE]&DJMK2_BULK_MASK_AUDIO_SOURCE_DECK_B) ?
				USB_PC_OUT : MK2_AUDIO_INPUT;
			if (eAudioSourceSelection!=m_pDevState->djConsoleMk2.eAudioSourceSelection[1]) {
				m_pDevState->djConsoleMk2.eAudioSourceSelection[1] = eAudioSourceSelection;
				m_pDevState->bStateChange = true;
			}

			// --- Lock State ---
			bLock = (pBuf[DJMK2_BULK_INDEX_LOCK_DECK]&DJMK2_BULK_MASK_LOCK_DECK_A)!=0;
			if (bLock!=m_pDevState->djConsoleMk2.bLockAudioSource[0])
			{
				m_pDevState->djConsoleMk2.bLockAudioSource[0] = bLock;
				m_pDevState->bStateChange = true;
			}

			bLock = (pBuf[DJMK2_BULK_INDEX_LOCK_DECK]&DJMK2_BULK_MASK_LOCK_DECK_B)!=0;
			if (bLock!=m_pDevState->djConsoleMk2.bLockAudioSource[1])
			{
				m_pDevState->djConsoleMk2.bLockAudioSource[1] = bLock;
				m_pDevState->bStateChange = true;
			}

			switch(pBuf[DJMK2_BULK_INDEX_HEADPHONE_SELECTION]&DJMK2_BULK_MASK_HEADPHONE_SELECTION)
			{
			case DJMK2_BULK_HEADPHONE_SELECTION_DECK_A:
				eHeadphoneSelection = DECK_A;
				break;
			case DJMK2_BULK_HEADPHONE_SELECTION_DECK_B:
				eHeadphoneSelection = DECK_B;
				break;
			case DJMK2_BULK_HEADPHONE_SELECTION_MIX:
				eHeadphoneSelection = MIX;
				break;
			case DJMK2_BULK_HEADPHONE_SELECTION_SPLIT:
				eHeadphoneSelection = SPLIT;
				break;
			default:
				// Use previous read state (the hardware may send a code which is ambiguous,
				// the driver should always report a valid state however).
				eHeadphoneSelection = m_pDevState->djConsoleMk2.eHeadphoneSelection;
				break;
			}

			if (eHeadphoneSelection!=m_pDevState->djConsoleMk2.eHeadphoneSelection)
			{
				m_pDevState->djConsoleMk2.eHeadphoneSelection = eHeadphoneSelection;
				m_pDevState->bStateChange = true;
			}
			break;
		}
	}

	delete pBuf;
	return NULL;
}


